#ifndef __NLL_COMMAND_H_INCLUDED__
#define __NLL_COMMAND_H_INCLUDED__

typedef enum {
  COMMAND_UNKNOWN = 0,
  COMMAND_NONE,
  COMMAND_LABEL,

  COMMAND_COMMENT,
  COMMAND_END,
  COMMAND_EXIT,
  COMMAND_LET,
  COMMAND_PRINT,
  COMMAND_GOTO,
  COMMAND_GONEXT,
  COMMAND_GOSUB,
  COMMAND_RETURN,
  COMMAND_LOOP,
  COMMAND_LOOPEND,
  COMMAND_FOR,
  COMMAND_NEXT,
  COMMAND_DISCARD,
  COMMAND_DIM,
  COMMAND_MEMORY,
  COMMAND_RUN,
  COMMAND_STOP,
  COMMAND_BREAK,
  COMMAND_STEP,
  COMMAND_CONTINUE,
  COMMAND_NEW,
  COMMAND_LIST,
  COMMAND_EDIT,
  COMMAND_WAIT,
  COMMAND_DUMP,

  COMMAND_LAST
} command_type_t;

typedef enum {
  COMMAND_ARG_UNKNOWN = 0,
  COMMAND_ARG_LABEL,
  COMMAND_ARG_VARIABLE,
  COMMAND_ARG_FORMULA,
  COMMAND_ARG_WORD,
} command_arg_type_t;

typedef struct command_ope {
  char *name;
  char *other_name;
  command_type_t type;
} *command_ope_t;

typedef struct command_arg {
  command_arg_type_t type;
  union {
    struct {
      label_t label;
      element_t args;
    } label;
    struct {
      variable_t variable;
    } variable;
    struct {
      element_t element;
    } formula;
    struct {
      char word[COMMAND_ARG_WORD_MAXLEN + 1];
    } word;
  } arg;
} *command_arg_t;

#define COMMAND_ARG_MAXNUM 4

struct command {
  struct command *next;
  command_ope_t ope;
  char *line;
  int errcode;
  element_t prefix;
  struct command_arg args[COMMAND_ARG_MAXNUM];
};

int command_check(void);
int command_search(char *p, char **endp, command_ope_t *opep);
int command_get(command_type_t type, command_ope_t *opep);
int command_alloc(command_t *commandp);
int command_free(command_t command);
int command_clear(command_t command);
int command_clean(command_t command);
int command_init(void);
int command_dump(command_t command);

#endif
