#include <stdio.h>
#include <stdlib.h>

#ifdef USE_NLLIBC
#include <nllibc.h>
#endif

#include "lib.h"

void nll_error_print(const char *filename, int line,
		     int errcode, const char *param)
{
  if (!param)
    param = "(?)";

  switch (errcode) {
  case NLL_ERRCODE_FILE_NOT_FOUND:
    printf("File not found: %s\n", param);
    break;
  case NLL_ERRCODE_LINE_BUFFER_OVER:
    printf("Line buffer overflow: %s\n", param);
    break;
  case NLL_ERRCODE_LINE_NOT_EMPTY:
    printf("Line not empty: %s\n", param);
    break;
  case NLL_ERRCODE_LINE_TOO_LONG:
    printf("Too long line: %s\n", param);
    break;
  case NLL_ERRCODE_LINE_NOT_FOUND:
    printf("Line not found: %s\n", param);
    break;
  case NLL_ERRCODE_VARIABLE_NOT_EMPTY:
    printf("Variable not empty: %s\n", param);
    break;
  case NLL_ERRCODE_VARIABLE_INVALID_NAME:
    printf("Invalid variable name: %s\n", param);
    break;
  case NLL_ERRCODE_VARIABLE_TOO_LONG:
    printf("Too long variable: %s\n", param);
    break;
  case NLL_ERRCODE_VARIABLE_BUFFER_OVER:
    printf("Variable buffer overflow: %s\n", param);
    break;
  case NLL_ERRCODE_VARIABLE_NOT_FOUND:
    printf("Variable not found: %s\n", param);
    break;
  case NLL_ERRCODE_VALUE_NOT_EMPTY:
    printf("Value not empty: %s\n", param);
    break;
  case NLL_ERRCODE_VALUE_BUFFER_OVER:
    printf("Value buffer overflow: %s\n", param);
    break;
  case NLL_ERRCODE_VALUE_INVALID_TYPE:
    printf("Invalid value type: %s\n", param);
    break;
  case NLL_ERRCODE_VALUE_CONST_VALUE:
    printf("Constant value: %s\n", param);
    break;
  case NLL_ERRCODE_VALUE_INVALID_VALUE:
    printf("Invalid value: %s\n", param);
    break;
  case NLL_ERRCODE_FORMULA_NOT_EMPTY:
    printf("Formula not empty: %s\n", param);
    break;
  case NLL_ERRCODE_FORMULA_BUFFER_OVER:
    printf("Calculation buffer overflow: %s\n", param);
    break;
  case NLL_ERRCODE_FORMULA_LESS_PARAMETER:
    printf("Less parameter in operand: %s\n", param);
    break;
  case NLL_ERRCODE_FORMULA_MUCH_PARAMETER:
    printf("Much parameter in operand: %s\n", param);
    break;
  case NLL_ERRCODE_FORMULA_DIV_ZERO:
    printf("Division by zero: %s\n", param);
    break;
  case NLL_ERRCODE_FORMULA_INVALID_OPERATOR:
    printf("Invalid operator: %s\n", param);
    break;
  case NLL_ERRCODE_FORMULA_INVALID_PARAMETER:
    printf("Invalid parameter: %s\n", param);
    break;
  case NLL_ERRCODE_FORMULA_UNSUPPORTED_OPERATOR:
    printf("Unsupported operator: %s\n", param);
    break;
  case NLL_ERRCODE_FORMULA_ARRAY_OVER:
    printf("Array size overflow: %s\n", param);
    break;
  case NLL_ERRCODE_FORMULA_UNKNOWN_FUNCTION:
    printf("Unknown function: %s\n", param);
    break;
  case NLL_ERRCODE_FORMULA_NULL_POINTER:
    printf("Null pointer access: %s\n", param);
    break;
  case NLL_ERRCODE_FORMULA_INVALID_POINTER:
    printf("Invalid pointer: %s\n", param);
    break;
  case NLL_ERRCODE_LABEL_NOT_EMPTY:
    printf("Label not empty: %s\n", param);
    break;
  case NLL_ERRCODE_LABEL_BUFFER_OVER:
    printf("Label buffer overflow: %s\n", param);
    break;
  case NLL_ERRCODE_LABEL_INVALID_NAME:
    printf("Invalid label name: %s\n", param);
    break;
  case NLL_ERRCODE_LABEL_TOO_LONG:
    printf("Too long label: %s\n", param);
    break;
  case NLL_ERRCODE_LABEL_NOT_FOUND:
    printf("Label not found: %s\n", param);
    break;
  case NLL_ERRCODE_LABEL_DUPLICATE:
    printf("Duplicate label: %s\n", param);
    break;
  case NLL_ERRCODE_POSITION_STACK_OVER:
    printf("Position stack overflow: %s\n", param);
    break;
  case NLL_ERRCODE_POSITION_LESS_PARAMETER:
    printf("Less position to return: %s\n", param);
    break;
  case NLL_ERRCODE_POSITION_DIFFERENT_TYPE:
    printf("Different position type: %s\n", param);
    break;
  case NLL_ERRCODE_POSITION_DIFFERENT_VARIABLE:
    printf("Different position variable: %s\n", param);
    break;
  case NLL_ERRCODE_COMMAND_NOT_EMPTY:
    printf("Command not empty: %s\n", param);
    break;
  case NLL_ERRCODE_COMMAND_BUFFER_OVER:
    printf("Command buffer overflow: %s\n", param);
    break;
  case NLL_ERRCODE_COMMAND_TOO_LONG:
    printf("Too long command: %s\n", param);
    break;
  case NLL_ERRCODE_COMMAND_NOT_FOUND:
    printf("Command not found: %s\n", param);
    break;
  case NLL_ERRCODE_STRING_NOT_EMPTY:
    printf("String not empty: %s\n", param);
    break;
  case NLL_ERRCODE_STRING_TOO_LONG:
    printf("Too long string: %s\n", param);
    break;
  case NLL_ERRCODE_STRING_BUFFER_OVER:
    printf("String buffer overflow: %s\n", param);
    break;
  case NLL_ERRCODE_ARRAY_BUFFER_OVER:
    printf("Array buffer overflow: %s\n", param);
    break;
  case NLL_ERRCODE_AREA_NOT_EMPTY:
    printf("Area not empty: %s\n", param);
    break;
  case NLL_ERRCODE_AREA_BUFFER_OVER:
    printf("Area buffer overflow: %s\n", param);
    break;
  case NLL_ERRCODE_AREA_TOO_LONG:
    printf("Too long area: %s\n", param);
    break;
  case NLL_ERRCODE_NLL_SYNTAX_ERROR:
    printf("Syntax error: %s\n", param);
    break;
  case NLL_ERRCODE_NLL_WORD_TOO_LONG:
    printf("Too long word: %s\n", param);
    break;
  case NLL_ERRCODE_NLL_NO_CONTINUE_POINT:
    printf("No continue point: %s\n", param);
    break;
  case NLL_ERRCODE_NLL_INVALID_COMMAND:
    printf("Invalid command: %s\n", param);
    break;

  case NLL_ERRCODE_UNKNOWN:
  default:
    printf("Unknown error: (%s:%d): %s\n", filename, line, param);
    break;
  }
}

void nll_error_exit(const char *filename, int line,
		    int errcode, const char *param)
{
  nll_error_print(filename, line, errcode, param);
  exit(1);
}

static int finished = 0;

void nll_finished_clear(void)
{
  finished = 0;
}

void nll_finish(void)
{
  finished = 1;
}

int nll_is_finished(void)
{
  return finished;
}
